import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.*;

public
class ClientRegistration extends Frame implements ActionListener {
    
    private Insets              mInsets = new Insets(3,3,3,3);

    private ClientTitlePanel       tp;
    private ClientInfoPanel        ip;
    private ClientStatusPanel      sp;

    private String                 serverName;
    private int                    portNo;
    private int                    failedCount = 1;

    private ClientRegSecuredComm   crsc;

    public
    ClientRegistration() { }

    protected void
    createGui() {
        setLayout(new BorderLayout());

        tp = new ClientTitlePanel("NPAC Register");
        add(tp, "North");

        ip = new ClientInfoPanel(this);
        add(ip, "Center");


        sp = new ClientStatusPanel(this);
        add(sp, "South");
    }

    // connect to the UPServer
    // to obtain the server host name and port number to be connected

    public boolean connectToUPServer(String host, int port) {
        System.out.println("Generating key pair....");
        ClientRegKeyPair crkp = new ClientRegKeyPair();

        try {
            Socket socket = new Socket(host, port);
            crsc = new ClientRegSecuredComm(socket, crkp);

            if (!crsc.init()) {
                System.err.println("Key verification failed");
                return false;
            } else {
                System.err.println("Key verification successful");
            }
            return true;
        } catch (Exception ex) {
            System.err.println("ClientRegistration Exception-> " + ex);
        }
        return false;
    }

   /**
    *  Handle action events from various controls
    *  Overrides Component.action().
    */
    public
    void actionPerformed(ActionEvent pe) {
        // User selected month from the choice
        String actionLabel = pe.getActionCommand();

        if (actionLabel.equals("Register")) {

            String userid = ip.getUsername();
            String passwd = ip.getPassword();
            String cpasswd = ip.getCPassword();
            String name = ip.getName();
            String org = ip.getOrganization();
            String email = ip.getEmail();

            if (userid.length() == 0) {
                sp.setText("Null account!  Try again!");
                return;
            }
            if (passwd.length() == 0) {
                sp.setText("Null password!  Try again!");
                return;
            }
            if (cpasswd.length() == 0) {
                sp.setText("Null password!  Try again!");
                return;
            }
            if (!passwd.equals(cpasswd)) {
                sp.setText("Unmatched passwords!  Try again!");
                return;
            }
            if (name.length() == 0) {
                sp.setText("Null account!  Try again!");
                return;
            }
            if (org.length() == 0) {
                sp.setText("Null account!  Try again!");
                return;
            }
            if (email.length() == 0) {
                sp.setText("Null account!  Try again!");
                return;
            }

            sp.setText("Connecting to NPAC UPServer");

            // Set user information and send it to server
            WFUserInfo wfui = new WFUserInfo(userid, passwd, name, org, email);
            crsc.ssend(wfui);

            // receive and verify the confirmed user information from UPServer
            WFUserInfo vwfui = (WFUserInfo)crsc.sreceive();
            if (vwfui != null) {
                if (vwfui.equals(wfui)) {
                    crsc.ssend(new WFMessage((int)1, ""));
                } else {
                    crsc.ssend(new WFMessage((int)0, ""));
                }
            } else {
                crsc.ssend(new WFMessage((int)0, ""));
            }

            WFMessage wfm = (WFMessage)crsc.sreceive();
            if (wfm != null) {
                if (wfm.type == 1) {
                    sp.setText("Registration done.");
                    System.err.println("Registration done.");
                    System.exit(0);
                }
            } 
            sp.setText("Registration failed!");
            System.err.println("Registration failed!");
            System.exit(1);
        } else if (actionLabel.equals("Clear")) {
            ip.clear();
        }
        validate();
    }

    /**
     *  Paint nice raized 3D background.
     *  Overrides Component.paint().
     */
    public void
    paint(Graphics pg)
    {
        pg.setColor(Color.black);
        pg.drawLine(1, 0, getBounds().width-2, 0);
        pg.drawLine(0, 1, 0, getBounds().height-2);
        pg.drawLine(1, getBounds().height-1, getBounds().width - 2, getBounds().height - 1);
        pg.drawLine(getBounds().width - 1, 1, getBounds().width - 1, getBounds().height - 2);

        pg.setColor(getBackground());
        //pg.draw3DRect(1, 1, getBounds().width - 3, getBounds().height - 3, true);
        pg.fill3DRect(1, 1, getBounds().width - 2, getBounds().height - 2, true);
    }

    /**
     *  For better looks, let Layout Manager leave some margin.
     */
	public
	Insets
	getInsets()
	{
	    return mInsets;
	}

    public static void
    main(String[] args) {

        if (args.length != 4) {
            System.err.println("java ClientRegistration -RHost hostname -RPort #");
            System.exit(1);
        }

        // parse the input arguments

        int i = 0;
        while (!args[i++].equals("-RHost") && (i < args.length)) {};
        String host = args[i];

        i = 0;
        while (!args[i++].equals("-RPort") && (i < args.length)) {};
        int port = Integer.parseInt(args[i]);

        System.out.println("Connecting to " + host + ":" + port);

        // initialization

        ClientRegistration sc = new ClientRegistration();

        if (!sc.connectToUPServer(host, port)) {
            System.err.println("Connection to " + host + ":" + port + " failed");
            System.exit(1);
        }
        System.out.println("Connected to " + host + ":" + port);

        sc.setSize(400, 300);
        sc.createGui();
        sc.show();
    }
}
